#	Copyright (c) 2009, 2010 Yves Adler <yves.adler@googlemail.com>
#
#	Permission is hereby granted, free of charge, to any person
#	obtaining a copy of this software and associated documentation
#	files (the "Software"), to deal in the Software without
#	restriction, including without limitation the rights to use,
#	copy, modify, merge, publish, distribute, sublicense, and/or sell
#	copies of the Software, and to permit persons to whom the
#	Software is furnished to do so, subject to the following
#	conditions:
#
#	The above copyright notice and this permission notice shall be
#	included in all copies or substantial portions of the Software.
#
#	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#	OTHER DEALINGS IN THE SOFTWARE.


module EvoSynth
	module Decoder

		#	:call-seq:
		#		Decoder.binary_to_real([1,1,0,1]) -> Float (between -1.0 and 1.0)
		#		Decoder.binary_to_real([1,1,0,1], 0.0) -> Float (between 0.0 and 1.0)
		#		Decoder.binary_to_real([1,1,0,1], -5.12, 5.12) -> Float (between -5.12 and 5.12)
		#
		# This converts a given array of binary values (either 1/0 or true/false) into
		# a Float in a given range.

		def Decoder.binary_to_real(binary, min_bound = -1.0, max_bound = 1.0)
			binary = binary.map do |bin|
				case bin
					when false then 0
					when true then 1
					else bin
				end
			end

			max_bound, min_bound = min_bound, max_bound if min_bound > max_bound

			range_size = max_bound.to_f - min_bound.to_f
			integer = Integer("0b#{binary.join}")
			max_integer = Integer("0b" + ("1" * binary.size))

			float = integer * (range_size / max_integer) + min_bound;
			float
		end

	end

end